#! /usr/bin/env python3
# -*- coding: utf-8 -*-
#

import argparse
import json
import os

from .importer import importrules
from .RulesTree import RulesTree


def main():
    parser = argparse.ArgumentParser(
        description='Awesome build manager to replace Makefiles. It allow very fast building!')
    parser.add_argument('--config', default='awesomebuild.json',
                        help='default: awesomebuild.json')
    parser.add_argument(
        'targets', nargs='*',
        help='defaults to the main rule defined in the config file'
    )
    args = parser.parse_args()

    configpath = os.path.abspath(
        (os.path.abspath(os.getcwd()) + '/'
         if args.config[0] != '/' else '') + args.config)

    if not os.path.isfile(configpath):
        raise FileNotFoundError('Config path invalid!')

    configdir = os.path.dirname(configpath)

    with open(configpath) as json_file:
        data = json.load(json_file)

    if 'rules' not in data:
        data['rules'] = {}

    data = importrules(data, configdir)

    targets = args.targets if len(args.targets) != 0 else [data['main']]

    rules = RulesTree(data, configdir)

    for t in targets:
        rules[t].run()


if __name__ == '__main__':
    main()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
#

import subprocess

from .Trigger import Trigger


def loadSingleMulti(data, n):
    if n in data:
        if not isinstance(data[n], list):
            return([data[n]])
        else:
            return(data[n])
    return([])


class Rule:
    def __init__(self, tree, name, data, configdir):
        self.tree = tree
        self.name = name
        self.cmd = loadSingleMulti(data, 'cmd')
        self.callBefore = loadSingleMulti(data, 'callBefore')
        self.callAfter = loadSingleMulti(data, 'callAfter')
        self.call = loadSingleMulti(data, 'call')
        self.configdir = configdir
        self.trigger = [
            Trigger(self.configdir, t) for
            t in loadSingleMulti(data, 'trigger')
        ]

    def show(self):
        print('name:', self.name)
        print('cmd:', self.cmd)
        print('callBefore:', self.callBefore)
        print('callAfter:', self.callAfter)
        print('call:', self.call)
        print('trigger:', self.trigger)
        print('-----')

    def run(self):
        run = False
        if self.execCallBefore():
            run = True
        if not run:
            if self.checkTriggers():
                run = True
        if run:
            print(self.name)
            self.executeCmds()
            self.execCall()
        self.execCallAfter()
        return(run)

    def executeCmds(self):
        for c in self.cmd:
            subprocess.call('cd ' + self.configdir + '; ' + c,
                            shell=True, stdout=subprocess.DEVNULL,
                            stderr=subprocess.DEVNULL
                            )

    def execCallBefore(self):
        run = False
        for cb in self.callBefore:
            if self.tree[cb].run():
                run = True
        return(run)

    def execCallAfter(self):
        for ca in self.callAfter:
            self.tree[ca].run()

    def execCall(self):
        for c in self.call:
            self.tree[c].run()

    def checkTriggers(self):
        run = False
        if len(self.trigger) == 0:
            run = True
        for t in self.trigger:
            if t.eval():
                run = True
        return(run)



from setuptools import setup

setup(
    name='AwesomeBuild',
    description='Awesome build manager to replace Makefiles. It allow very fast building!',
    version='2.0.1',
    author='Raphael Jacob',
    author_email='r.jacob2002@gmail.com',
    url='https://github.com/ski7777/AwesomeBuild',
    license='GPLv3',
    packages=['AwesomeBuild'],
    entry_points={
        'console_scripts': [
            'AwesomeBuild = AwesomeBuild.__main__:main'
        ]
    }
)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
#

import glob
import json
import os


def importrules(data, configdir):
    importrules = []
    if 'import' in data:
        if isinstance(data['import']['rules'], dict):
            importrules = data['import']['rules']
        else:
            importrules += data['import']['rules']
    for ir in importrules:
        irr = {}
        ipath = os.path.abspath(os.path.join(configdir, ir['value']))
        if ir['type'] == 'directory':
            for irf in glob.glob(ipath + '/*.json'):
                with open(irf) as json_file:
                    irr.update(json.load(json_file))
        if ir['type'] == 'file':
            with open(ipath) as json_file:
                irr = json.load(json_file)
        if ir['rules'] == '*':
            data['rules'].update(irr)
        else:
            for irrl in ir['rules']:
                data['rules'][irrl] = irr[irrl]
    return(data)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
#

import json
import os


class Trigger:
    def __init__(self, configdir, triggerconfig):
        self.configdir = configdir
        self.triggerconfig = triggerconfig
        self.triggerstatus = TriggerStatus(
            os.path.abspath(
                os.path.join(self.configdir, 'awesomestatus.json')
            )
        )
        self.value = self.triggerconfig['value']
        self.eval, ispath = {
            'file': {
                'changed': (self.fileChangedTrigger, True),
                'exist': (self.fileexistTrigger, True),
                'not exist': (self.fileNotexistTrigger, True)
            },
            'directory': {
                'changed': (self.directoryChangedTrigger, True),
                'exist': (self.directoryexistTrigger, True),
                'not exist': (self.directoryNotexistTrigger, True)
            }
        }[self.triggerconfig['type']][self.triggerconfig['subtype']]
        if ispath:
            self.relpath = self.value
            self.abspath = os.path.abspath(
                os.path.join(self.configdir, self.value)
            )

    def fileChangedTrigger(self):
        ts = self.triggerstatus[self.relpath]
        if not os.path.isfile(self.abspath):
            return(True)
        cs = os.popen('cat ' + self.abspath + ' | sha512sum').read()
        self.triggerstatus[self.relpath] = cs
        if ts is None:
            return(True)
        if ts != cs:
            return(True)
        return(False)

    def fileexistTrigger(self):
        return(os.path.isfile(self.abspath))

    def fileNotexistTrigger(self):
        return(not os.path.isfile(self.abspath))

    def directoryChangedTrigger(self):
        ts = self.triggerstatus[self.relpath]
        if not os.path.isdir(self.abspath):
            return(True)
        cs = os.popen(
            'cd ' + self.configdir
            + '; ls -RlgAGi --time-style=+%s '
            + self.relpath + ' | sha512sum'
        ).read()
        self.triggerstatus[self.relpath] = cs
        if ts is None:
            return(True)
        if ts != cs:
            return(True)
        return(False)

    def directoryexistTrigger(self):
        return(os.path.isdir(self.abspath))

    def directoryNotexistTrigger(self):
        return(not os.path.isdir(self.abspath))


class TriggerStatus:
    def __init__(self, path):
        self.abspath = path

    def __getitem__(self, key):
        if os.path.isfile(self.abspath):
            with open(self.abspath) as json_file:
                data = json.load(json_file)
            if key in data:
                return(data[key])
        return(None)

    def __setitem__(self, key, value):
        data = {}
        if os.path.isfile(self.abspath):
            with open(self.abspath) as json_file:
                data = json.load(json_file)
        data[key] = value
        with open(self.abspath, 'w') as json_file:
            json.dump(data, json_file, indent=4, sort_keys=True)
